#!/usr/bin/env python
# -*- coding: utf-8 -*-

from xml.dom.minidom import parse
from os.path import exists
from os.path import getmtime
import sys, codecs, re
from optparse import OptionParser

commentmatch = re.compile('^\s*(.+)\s*')
ampmatch = re.compile('&')

citydic = {}

class restaurant:
    def __init__(self, info):
        self.info = info
    def __lt__(self, rhs):
        if self.info['city'] < rhs.info['city']:
            return True
        elif self.info['city'] == rhs.info['city'] and self.info['name'] < rhs.info['name']:
            return True
        else:
            return False
    def getcity(self):
        return self.info['city']
    def htmldump(self, f, idt):
        closed = False
        if self.info.has_key('obsolete') and self.info['obsolete'] == 'yes':
            closed = True

        idt2 = idt * 2
        f.write(idt + '<li>\n')
        if closed:
            f.write(idt2 + '<del>')
        f.write(idt2)
        if self.info.has_key('url'):
            f.write('<a href="' + self.info['url'] + '">' + self.info['name'] + '</a>')
        else:
            f.write(self.info['name'])
        f.write(' (' + self.info['genre'])
        if self.info.has_key('subgenre'):
            f.write(' / ' + self.info['subgenre'])
        f.write(')<br />\n')
        if closed:
            f.write(idt2 + '</del>')
        f.write(idt2 + self.info['address'] + ', ' + self.info['city'] + ', ')
        f.write(self.info['state'] + ' ' + self.info['zip'])
        f.write('<a href="https://maps.google.com/maps?q=')
        latitude = float(self.info['latitude'].encode('ascii', 'strict'))
        f.write(str(abs(latitude)))
        if latitude < 0:
            f.write('S')
        else:
            f.write('N')
        longitude = float(self.info['longitude'].encode('ascii', 'strict'))
        f.write(',' + str(abs(longitude)))
        if longitude < 0:
            f.write('W')
        else:
            f.write('E')
        f.write('(' + self.info['name'] + ')">')
        f.write('(map)</a>, ' + self.info['phone'] + '\n' + idt2 + '<br />\n')

        if closed:
            f.write(idt2 + u'注: 現時点で閉店の模様<br />\n')

        if self.info.has_key('comment'):
            m = commentmatch.search(self.info['comment'])
            f.write(idt2 + m.group(1) + '\n' + idt2 + '<br />\n')

        f.write(idt2 + 'Photo:\n')
        for pinfo in self.info['photo']:
            f.write(idt2 + '<a href="' + pinfo['url'] + '">' + pinfo['date'] + '</a>')
            if pinfo == self.info['photo'][-1]:
                f.write('\n')
            else:
                f.write(',\n')

        if len(self.info['thumbnail']) > 0:
            f.write(idt2 + '<ul class="thumbnail">\n')
            for tminfo in self.info['thumbnail']:
                tn_path = tminfo['url'].replace('http://www.jinmei.org/', '/')
                idx_path = tminfo['img'].replace('http://www.jinmei.org/', '/')
                f.write(idt2 + idt + '<li><a class="defaultcolor" href="' + tn_path + '">')
                f.write('<img src="' + idx_path + '" alt="" /></a></li>\n')
            f.write(idt2 + '</ul>\n')

        if len(self.info['links']) > 0:
            if len(self.info['thumbnail']) == 0:
                f.write(idt2 + '<br />\n')
            f.write(idt2 + 'Links:\n')
            for link in self.info['links']:
                if (link['type'] == 'yelp'):
                    f.write(idt2 + 'Yelp:')
                    f.write(' (<a href="' + link['url'] + '">')
                    f.write('reviews</a> / ')
                    f.write('<span id="yelpinfo-')
                    yid = self.info['phone'].replace('-', '')
                    f.write(yid + '"><a href="')
                    f.write('javascript:YelpInfo.getRatingByPhonenum(')
                    f.write(yid + ')">see rating</a></span>)')
                else:
                    f.write(idt2 + '<a href="' + link['url'] + '">')
                    f.write(link['type'] + '</a>')
                if link == self.info['links'][-1]:
                    f.write('\n')
                else:
                    f.write(',\n')
        f.write(idt + '</li>\n')
    def dump(self):
        return self.info['name'] + ' in ' + self.info['city']

def get_restaurants(xmlfile):
    obj = parse(xmlfile)
    return obj.getElementsByTagName('restaurants')[0]

def make_infoset(node, info):
    if node.nodeType == node.ELEMENT_NODE:
        for child in node.childNodes:
            if child.nodeType == node.TEXT_NODE:
                info[node.tagName.encode('ascii', 'strict')] = child.data
                break

def make_resinfo(node, resinfo):
    if node.nodeType == node.ELEMENT_NODE:
        if node.tagName == 'photo':
            photoinfo = {}
            for child in node.childNodes:
                make_infoset(child, photoinfo)
            resinfo['photo'].append(photoinfo)
        elif node.tagName == 'thumbnail':
            thumbinfo = {}
            for child in node.childNodes:
                make_infoset(child, thumbinfo)
            resinfo['thumbnail'].append(thumbinfo)
        elif node.tagName == 'links':
            for child in node.childNodes:
                if child.nodeType == child.ELEMENT_NODE and child.tagName == 'link':
                    linkinfo = {}
                    for gchild in child.childNodes:
                        make_infoset(gchild, linkinfo)
                    resinfo['links'].append(linkinfo)
        else:
            for child in node.childNodes:
                if child.nodeType == node.TEXT_NODE:
                    global citydic
                    d = ampmatch.sub('&amp;', child.data)
                    tn = node.tagName.encode('ascii', 'strict')
                    resinfo[tn] = d
                    if tn == 'city':
                        if citydic.has_key(d):
                            citydic[d] += 1
                        else:
                            citydic[d] = 1
                    break

def make_list(node, reslist):
    if node.nodeType == node.ELEMENT_NODE:
        if node.tagName == 'restaurant':
            resinfo = {}
            resinfo['photo'] = []
            resinfo['thumbnail'] = []
            resinfo['links'] = []
            for child in node.childNodes:
                make_resinfo(child, resinfo)
            res = restaurant(resinfo)
            reslist.append(res)
        else:
            for child in node.childNodes:
                make_list(child, reslist)

def printheader(f, jintro, eintro, nrestaurant, local):
    if local:
        csspath = u'../css/'
        jspath = u'./'
    else:
        csspath = u'/css/'
        jspath = u'/js/'
    f.write(u'''<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
    "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="ja" lang="ja">
<head>
<meta http-equiv="Content-Type" content="application/xhtml+xml; charset=UTF-8" />
<script src="''' + jspath + u'''yelpinfo.js" type="text/javascript"></script>
<title>A list of SFBA restaurants</title>
<link rel="stylesheet" type="text/css" href="''' + csspath + u'''style.css" />
<link rel="stylesheet" type="text/css" href="''' + csspath + u'''thumbnail.css" />
</head>

<body>
<div class="mainpage">
<h1>San Francisco Bay Areaのレストランリスト</h1>
<hr />
<p>\n''')
    f.write('  ' + jintro + '\n</p>\n<p>\n  ' + eintro + '\n</p>\n')
    f.write('Last Updated: <!--#echo var="LAST_MODIFIED"-->.<br />\n')
    f.write(str(nrestaurant) + ' restaurants are listed as of that date.</p>\n')
    f.write('<hr />\n\n')

def printfooter(f):
    f.write(u'''<hr />
<address>
Email: jinmei at wide dot ad dot jp<br />
</address>
</div>

<div class="rightbar">

<h2>Search</h2>

<form action="https://www.google.com/search">
  <p>
  <input type="hidden" name="ie" value="UTF-8" />
  <input type="hidden" name="hl" value="ja" />
  <input type="hidden" name="domains" value="www.jinmei.org" />
  <input type="hidden" name="sitesearch" value="www.jinmei.org" />
  <input type="text" name="q" size="15" maxlength="256" value="" />
  <br />
  <input type="submit" name="btnG" value="サイト内検索" />
  <br />
  <a class="defaultcolor" href="https://www.google.com/">
  <img src="https://www.google.com/logos/Logo_40wht.gif" width="64"
  height="26" style="border: none;" alt="Google"  /></a>
  </p>
</form>
''')
    f.write('''<!--#include virtual="/adsense-include.html"-->
<!--#include virtual="/ga-include.html"-->
</body>
</html>\n''')

def printindex(f):
    f.write('<h3>City Index</h3>\n<p>\n')

    citylist = citydic.keys()
    citylist.sort()
    for c in citylist:
        f.write('<a href="#' + c +'">' + c + '</a>')
        if c != citylist[-1]:
            f.write(' / ')
    f.write('\n</p><hr />\n')

if __name__ == '__main__':
    usage = '''usage: %prog [options] inputfile outputfile'''
    parser = OptionParser(usage = usage)
    parser.add_option("-f", "--force", action="store_true",
                  dest="force", default=False,
                  help="force to output [default: %default]")
    parser.add_option("-l", "--local", action="store_true",
                  dest="local", default=False,
                  help="use local links for testing [default: %default]")
    parser.add_option("-q", "--quiet", action="store_true",
                  dest="quiet", default=False,
                  help="less verbose [default: %default]")
    (options, args) = parser.parse_args()

    if len(args) < 2:
        parser.error('missing argument')

    if not options.force and exists(args[1]) and getmtime(args[1]) >= getmtime(args[0]):
        if not options.quiet:
            sys.stderr.write('input file is older than output, exiting\n')
        sys.exit(0)

    xmlobj = parse(args[0])
    restaurants = xmlobj.getElementsByTagName('restaurants')[0]
    node = xmlobj.getElementsByTagName('japaneseintro')[0]
    for child in node.childNodes:
        if child.nodeType == node.TEXT_NODE:
            jintro = child.data
            m = commentmatch.search(jintro)
            jintro = m.group(1)
    node = xmlobj.getElementsByTagName('englishintro')[0]
    for child in node.childNodes:
        if child.nodeType == node.TEXT_NODE:
            eintro = child.data
            m = commentmatch.search(eintro)
            eintro = m.group(1)

    f = codecs.open(args[1], 'w', 'utf-8')
    if restaurants:
        reslist = []
        make_list(restaurants, reslist)
        printheader(f, jintro, eintro, len(reslist), options.local)
        printindex(f)
        reslist.sort()
        currentcity = ''
        for r in reslist:
            city = r.getcity()
            if city != currentcity:
                if currentcity != '':
                    f.write('</ul>\n')
                f.write('<h2><a name="' + city + '">' + city + '</a> (')
                f.write(str(citydic[city]) + ' restaurant')
                if citydic[city] != 1:
                    f.write('s')
                f.write(')</h2>\n')
                f.write('<ul>\n')
                currentcity = city
            r.htmldump(f, '  ')
        f.write('</ul>\n')
    printfooter(f)
